/* LICAR
-- This code uses an HC-SR04 module to measure the distance between an overtaking car and you. With use of the time interval between 2 readings we can determine the speed of the other car.
-- Licar can inform the driver, with use of an LED, if a car is going to overtake them with a huge speed difference so the driver is allerted and can be aware of the dangerous situation.
-- Licar is made by: 
----Dennis Bitoiu
----Ewoud Janus
----Gracy Nedev
----Thijs Deunk
----Daniel Mitev
----Marnix Lueb
This is a project we made for the Smart Environments project in CreaTe module 2, assisted by Andreas Kamilaris and Antoine Moghaddar. 
date: 11 November 2020 till 28 January 2021
*/ 

#include <SR04.h>

#define TRIG_PIN 8   //the send signal pin is connected to pin 8
#define ECHO_PIN 7   //the receive signal pin is connected to pin 7 
SR04 sr04 = SR04(ECHO_PIN, TRIG_PIN);
long thisDistance;
float previousDistance;
float measuredDistance;
float speedCar1;
float speedCar2;
boolean recklessDriver;
unsigned long time;  

void setup() {
  //Serial.begin(9600);   All the commented serial code in this code can be used to see in the serial monitor what data you are getting, ideal for troubleshooting.
  //delay(1000);
  speedCar1 = 10;   //the speed of our car is 10km/h (ideally we would have used a GPS module to detrmine our speed, but ours didn't work so we have to set it to a default value)
  recklessDriver = false;
  pinMode(2, OUTPUT);   //an LED is connected to pin 2
}

void loop() {
  thisDistance = sr04.Distance();
  //Serial.print(thisDistance);
  //Serial.println(" cm");
  measuredDistance = previousDistance - thisDistance;   //with this simple formula we can determine the speed of the other car
  speedCar2 = measuredDistance / 0.1;   //we use 0.1 because we measure every 100ms (0.1s). 
  //Serial.print(previousDistance);
  //Serial.println(" cm");
  previousDistance = thisDistance;

  //Serial.print(speedCar2 / 100);
  //Serial.println(" m/s");

  //Serial.print(speedCar2 / 100 * 3.6);
  //Serial.println(" km/u");

  if (speedCar1 + 50 <= (speedCar2 / 100 * 3.6)) {   //this determines if the other car is going 50 or more kilometers an hour faster than our car.
    recklessDriver = true;
  } else {
    recklessDriver = false;
  }

  if (recklessDriver) {
    digitalWrite(2, HIGH);
    time = millis();   //this resets the timer meganism we use in the else statement
  } else {
    if (millis() >= time + 5000) {   //this line lets the LED only turn off after 5 seconds of it being on to make shure the driver can see it.
      digitalWrite(2, LOW);
    }
  }
  delay(100);   //we measure every 100ms (0.1s) to easily determine the timeinterval between 2 measurements
}
